@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- encoding: utf-8 -*-
 """
-基于unittest开发的测试框架，更友好、更灵活的编写、管理与运行测试，生成更加美观的独立单文件HTML报告。内置参数化测试数据存取方案，省去设计的烦恼，节省更多的时间，从而更快的投入到编写用例阶段
+更友好、更灵活的编写、管理与运行测试，生成更加美观的独立单文件HTML报告。内置参数化测试数据存取方案，省去设计的烦恼，节省更多的时间，从而更快的投入到编写用例阶段
 
  * 使用装饰器Test来标记测试方法更加灵活
  * 支持命名测试方法且不与方法的doc string（文档字符串）冲突
@@ -17,11 +17,11 @@
  * 支持灵活控制测试失败自动截图并附加到测试报告中
  * 支持page object模式，内置一套易于维护的解决方案
  * 驱动管理器（DRIVER_MANAGER）更加便捷的管理打开的驱动会话
- * 对selenium、appium、minium（微信小程序自动化测试库）以及WinAppDriver（微软官方提供的一款用于做Window桌面应用程序的界面（UI）自动化测试工具）做了底层集成支持
+ * 提供selenium、appium、playwright、minium（微信小程序自动化测试库）以及WinAppDriver（微软官方提供的一款用于做Window桌面应用程序的界面（UI）自动化测试工具）的page object的实现方案
 
 
 Simple usage:
-
+```py
     import os
 
     from stest import settings
@@ -123,6 +123,7 @@ Simple usage:
 
     if __name__ == '__main__':
         CalculationTest.run_test()
+```
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
