@@ -4,7 +4,12 @@ Created on 2019年12月5日
 
 @author: siwenwei
 '''
+import re
 import html
+import json
+import warnings
+import traceback
+from stest.core.errors import StestWarning
 from ..html import elements
 from .piechart import PieChart
 
@@ -12,33 +17,41 @@ from .piechart import PieChart
 class SummaryTable(elements.Table):
 
     FIXED_WIDTH = "fixed-width"
+    TONGJI_CSS_CLASS = ["statistics"]
     DEFAULT_CSS_CLASSES = ["seven-table", "summary"]
     PIE_CHART_FAIL_LABEL = "失败"
     PIE_CHART_FAIL_COLOR = "red"
-    PIE_CHART_FAIL_CSS_CLASSES = {"pie_chart_css_class": "fail", "legend_css_class": "fail-legend", "legend_text_css_class": "fail-legend-text"}
+    PIE_CHART_FAIL_CSS_CLASSES = {"pie_chart_css_class": "fail",
+                                  "legend_css_class": "fail-legend", "legend_text_css_class": "fail-legend-text"}
     PIE_CHART_BLOCK_LABEL = "阻塞"
     PIE_CHART_BLOCK_COLOR = "sandybrown"
-    PIE_CHART_BLOCK_CSS_CLASSES = {"pie_chart_css_class": "block", "legend_css_class": "block-legend", "legend_text_css_class": "block-legend-text"}
+    PIE_CHART_BLOCK_CSS_CLASSES = {"pie_chart_css_class": "block",
+                                   "legend_css_class": "block-legend", "legend_text_css_class": "block-legend-text"}
 
     PIE_CHART_ERROR_LABEL = "异常"
     PIE_CHART_ERROR_COLOR = "blueviolet"
-    PIE_CHART_ERROR_CSS_CLASSES = {"pie_chart_css_class": "error", "legend_css_class": "error-legend", "legend_text_css_class": "error-legend-text"}
+    PIE_CHART_ERROR_CSS_CLASSES = {"pie_chart_css_class": "error",
+                                   "legend_css_class": "error-legend", "legend_text_css_class": "error-legend-text"}
 
     PIE_CHART_SUCCESS_LABEL = "通过"
     PIE_CHART_SUCCESS_COLOR = "green"
-    PIE_CHART_SUCCESS_CSS_CLASSES = {"pie_chart_css_class": "success", "legend_css_class": "success-legend", "legend_text_css_class": "success-legend-text"}
+    PIE_CHART_SUCCESS_CSS_CLASSES = {"pie_chart_css_class": "success",
+                                     "legend_css_class": "success-legend", "legend_text_css_class": "success-legend-text"}
 
     PIE_CHART_SKIP_LABEL = "skip"
     PIE_CHART_SKIP_COLOR = "#c0c0c0"
-    PIE_CHART_SKIP_CSS_CLASSES = {"pie_chart_css_class": "skip", "legend_css_class": "skip-legend", "legend_text_css_class": "skip-legend-text"}
+    PIE_CHART_SKIP_CSS_CLASSES = {"pie_chart_css_class": "skip",
+                                  "legend_css_class": "skip-legend", "legend_text_css_class": "skip-legend-text"}
 
     PIE_CHART_XFAIL_LABEL = "预期失败"
     PIE_CHART_XFAIL_COLOR = "#9acd32"
-    PIE_CHART_XFAIL_CSS_CLASSES = {"pie_chart_css_class": "xfail", "legend_css_class": "xfail-legend", "legend_text_css_class": "xfail-legend-text"}
+    PIE_CHART_XFAIL_CSS_CLASSES = {"pie_chart_css_class": "xfail",
+                                   "legend_css_class": "xfail-legend", "legend_text_css_class": "xfail-legend-text"}
 
     PIE_CHART_XPASS_LABEL = "unexpected passes"
     PIE_CHART_XPASS_COLOR = "#a52a2a"
-    PIE_CHART_XPASS_CSS_CLASSES = {"pie_chart_css_class": "xpass", "legend_css_class": "xpass-legend", "legend_text_css_class": "xpass-legend-text"}
+    PIE_CHART_XPASS_CSS_CLASSES = {"pie_chart_css_class": "xpass",
+                                   "legend_css_class": "xpass-legend", "legend_text_css_class": "xpass-legend-text"}
 
     SVG_BOX_CSS_CLASSES = ["svg-box"]
 
@@ -58,7 +71,8 @@ class SummaryTable(elements.Table):
     def _build_header(self):
         row = elements.TR()
         colspan = 3 if self.pie_chart_info else 2
-        task_title_cell = elements.TD(html.escape(self.title, False)).add_css_class(self.FIXED_WIDTH).set_attr("colspan", colspan)
+        task_title_cell = elements.TD(html.escape(self.title, False)).add_css_class(
+            self.FIXED_WIDTH).set_attr("colspan", colspan)
         row.append_child(task_title_cell)
         self.thead.append_child(row)
 
@@ -69,6 +83,7 @@ class SummaryTable(elements.Table):
         self._set_start_time(self.start_time)
         self._set_finish_time(self.finish_time)
         self._set_task_description(self.task_description)
+        self._set_testcase_statistics(self.pie_chart_info)
 
     def _set_task_number(self, task_number):
 
@@ -82,13 +97,20 @@ class SummaryTable(elements.Table):
             r = 100
             pass_count, fail_count, block_count, error_count, skip_count, xfail_count, xpass_count = self.pie_chart_info
             parts = [
-                PieChart.build_part(pass_count, self.PIE_CHART_SUCCESS_LABEL, self.PIE_CHART_SUCCESS_COLOR, **self.PIE_CHART_SUCCESS_CSS_CLASSES),
-                PieChart.build_part(fail_count, self.PIE_CHART_FAIL_LABEL, self.PIE_CHART_FAIL_COLOR, **self.PIE_CHART_FAIL_CSS_CLASSES),
-                PieChart.build_part(block_count, self.PIE_CHART_BLOCK_LABEL, self.PIE_CHART_BLOCK_COLOR, **self.PIE_CHART_BLOCK_CSS_CLASSES),
-                PieChart.build_part(error_count, self.PIE_CHART_ERROR_LABEL, self.PIE_CHART_ERROR_COLOR, **self.PIE_CHART_ERROR_CSS_CLASSES),
-                PieChart.build_part(skip_count, self.PIE_CHART_SKIP_LABEL, self.PIE_CHART_SKIP_COLOR, **self.PIE_CHART_SKIP_CSS_CLASSES),
-                PieChart.build_part(xfail_count, self.PIE_CHART_XFAIL_LABEL, self.PIE_CHART_XFAIL_COLOR, **self.PIE_CHART_XFAIL_CSS_CLASSES),
-                PieChart.build_part(xpass_count, self.PIE_CHART_XPASS_LABEL, self.PIE_CHART_XPASS_COLOR, **self.PIE_CHART_XPASS_CSS_CLASSES),
+                PieChart.build_part(pass_count, self.PIE_CHART_SUCCESS_LABEL,
+                                    self.PIE_CHART_SUCCESS_COLOR, **self.PIE_CHART_SUCCESS_CSS_CLASSES),
+                PieChart.build_part(fail_count, self.PIE_CHART_FAIL_LABEL,
+                                    self.PIE_CHART_FAIL_COLOR, **self.PIE_CHART_FAIL_CSS_CLASSES),
+                PieChart.build_part(block_count, self.PIE_CHART_BLOCK_LABEL,
+                                    self.PIE_CHART_BLOCK_COLOR, **self.PIE_CHART_BLOCK_CSS_CLASSES),
+                PieChart.build_part(error_count, self.PIE_CHART_ERROR_LABEL,
+                                    self.PIE_CHART_ERROR_COLOR, **self.PIE_CHART_ERROR_CSS_CLASSES),
+                PieChart.build_part(skip_count, self.PIE_CHART_SKIP_LABEL,
+                                    self.PIE_CHART_SKIP_COLOR, **self.PIE_CHART_SKIP_CSS_CLASSES),
+                PieChart.build_part(xfail_count, self.PIE_CHART_XFAIL_LABEL,
+                                    self.PIE_CHART_XFAIL_COLOR, **self.PIE_CHART_XFAIL_CSS_CLASSES),
+                PieChart.build_part(xpass_count, self.PIE_CHART_XPASS_LABEL,
+                                    self.PIE_CHART_XPASS_COLOR, **self.PIE_CHART_XPASS_CSS_CLASSES),
             ]
             pie = PieChart(parts, width, height, cx, cy, r)
             pie_chart_column = elements.TD().set_attr("rowspan", 6)
@@ -136,11 +158,40 @@ class SummaryTable(elements.Table):
         row, title_col, value_col = self._build_row(title, finish_time)
         self.tbody.append_child(row)
 
+    def _set_testcase_statistics(self, resultlist):
+
+        title = "用例统计"
+        tj = {"总计": 0, "通过": 0, "失败": 0, "阻塞": 0, "异常": 0,
+              "skip": 0, "预期失败": 0, "unexpected passes": 0}
+        if resultlist:
+            total = 0
+            for one in resultlist:
+                total = total + one
+            tj["总计"] = total
+            tj["通过"] = resultlist[0]
+            tj["失败"] = resultlist[1]
+            tj["阻塞"] = resultlist[2]
+            tj["异常"] = resultlist[3]
+            tj["skip"] = resultlist[4]
+            tj["预期失败"] = resultlist[5]
+            tj["unexpected passes"] = resultlist[6]
+
+        row, title_col, value_col = self._build_row(title, "")
+        row.add_css_class(*self.TONGJI_CSS_CLASS)
+        for k, v in tj.items():
+            box = elements.Span()
+            name = elements.Span(html.escape(k, False))
+            value = elements.Span(html.escape(str(v), False))
+            box.append_child(name).append_child(value)
+            value_col.append_child(box)
+        self.tbody.append_child(row)
+
 
 class ReportTable(elements.Table):
 
     DEFAULT_CSS_CLASSES = ["seven-table", "details"]
-    FIXED_HEADER_TITLES = ["测试点 / 测试用例", "总计", "通过", "失败", "阻塞", "异常", "skip", "预期失败", "unexpected passes"]
+    FIXED_HEADER_TITLES = ["测试点 / 测试用例", "总计", "通过", "失败",
+                           "阻塞", "异常", "skip", "预期失败", "unexpected passes"]
     TESTPOINT_ID_PREFIX = "testpoint_"
     TESTCASE_ID_PREFIX = "testcase_"
     ID_SEP = "."
@@ -167,10 +218,11 @@ class ReportTable(elements.Table):
     SCREENSHOT_OF_TEST_FAILURE = "screenshot-of-test-failure"
     TESTCASE_SHOW_INFO_LAYER = 'testcase-show-info-layer'
 
-    def __init__(self, testpoints=[]):
+    def __init__(self, testpoints=[], settings=None):
         super().__init__()
         self._header_titles = []
         self._testpoints = testpoints
+        self.settings = settings
         self._header_titles.extend(self.FIXED_HEADER_TITLES)
         self.append_child(elements.Thead(), elements.Tbody())
         self.add_css_class(*self.DEFAULT_CSS_CLASSES)
@@ -224,18 +276,102 @@ class ReportTable(elements.Table):
             fs = elements.Fieldset(html.escape(k, False)).add_css_class("fieldset-normal")
             if not isinstance(v, str):
                 v = str(v)
-            fs.append_child(elements.Pre(html.escape(v, False)).add_css_class("fieldset-normal-content"))
+            fs.append_child(elements.Pre(html.escape(v, False)
+                                         ).add_css_class("fieldset-normal-content"))
             el_content.append_child(fs)
         return el_fs
 
-    def __build_extra_info_area(self, extra_info, label=""):
+    @classmethod
+    def to_printable(cls, e):
+        error_message = "发生异常，将使用默认方法\n"
+        error_message += "异常详细信息:\n"
+        error_message += traceback.format_exc()
+        return error_message
 
-        namemaps = {
-            "author": "编写者",
-            "editors": "修改者",
-            "last_modifyied_by": "最后修改者",
-            "last_modified_time": "最近修改时间",
-        }
+    def format_param_name(self, param, alias, fmt="{param}"):
+
+        var_name = "{param}"
+        var_alias = "{alias}"
+        default_fmt = "{param}"
+        regex = re.compile(r"{}|{}".format(var_name, var_alias))
+
+        def repalce(m: re.Match):
+            mstr = m.group()
+            if mstr == var_name:
+                return param
+            elif mstr == var_alias:
+                return alias
+            else:
+                return ""
+        if regex.search(fmt):
+            return regex.sub(repalce, fmt)
+        else:
+            return regex.sub(repalce, default_fmt)
+
+    def get_param_name_html_text(self, param):
+
+        empty_str = ""
+        default_fmt = "{param}"
+        fmt = getattr(self.settings, "TEST_PARAM_NAME_FORMAT_STRING", None)
+        aliasinfo = getattr(self.settings, "TEST_PARAM_ALIAS", {})
+
+        v = aliasinfo.get(param, None)
+        if isinstance(v, str):
+            alias = v
+        elif isinstance(v, dict):
+            alias = v.get("alias", empty_str)
+        else:
+            alias = empty_str
+
+        if not isinstance(alias, str):
+            alias = empty_str
+
+        if callable(fmt):
+            try:
+                text = fmt(param, alias)
+            except Exception as err:
+                msg = self.to_printable(err)
+                warnings.warn(msg, category=StestWarning, stacklevel=2)
+                text = self.format_param_name(param, alias, fmt)
+                text = html.escape(text, False)
+        else:
+            if not isinstance(fmt, str) or alias.strip() == empty_str:
+                fmt = default_fmt
+            text = self.format_param_name(param, alias, fmt)
+            text = html.escape(text, False)
+        return text
+
+    def __test_param_value_formatter(self, param, value):
+        try:
+            fstr = json.dumps(value, ensure_ascii=False, indent=4)
+        except Exception:
+            fstr = str(value)
+        return fstr
+
+    def get_param_value_html_text(self, param, value):
+
+        dict_or_callable = getattr(self.settings, 'TEST_PARAM_VALUE_FORMATTER', None)
+        if isinstance(dict_or_callable, dict):
+            formatter = dict_or_callable.get(param, None)
+        elif callable(dict_or_callable):
+            formatter = dict_or_callable
+        else:
+            formatter = None
+        if callable(formatter):
+            try:
+                rv = formatter(param, value)
+            except Exception as err:
+                msg = self.to_printable(err)
+                warnings.warn(msg, category=StestWarning, stacklevel=2)
+                return html.escape(self.__test_param_value_formatter(param, value), False)
+            if not isinstance(rv, str):
+                return html.escape(self.__test_param_value_formatter(param, value), False)
+            else:
+                return rv
+        else:
+            return html.escape(self.__test_param_value_formatter(param, value), False)
+
+    def __build_extra_info_area(self, extra_info, label=""):
 
         el_fs, el_item, el_title, el_content = self.__build_fieldset()
         el_title_name = elements.Span(html.escape(label, False))
@@ -246,11 +382,10 @@ class ReportTable(elements.Table):
         for k, v in extra_info.items():
             if not v:
                 continue
-            v = ",".join([str(one) for one in v]) if isinstance(v, (list, tuple)) else v
-            v = v if isinstance(v, str) else str(v)
-            # el_dl, el_dt, el_dd = self.__build_seven_dl(html.escape(namemaps.get(k, k), False), elements.Pre(html.escape(v, False)))
-            el_dl = elements.Fieldset(html.escape(namemaps.get(k, k), False)).add_css_class("fieldset-normal")
-            el_dl.append_child(elements.Pre(html.escape(v, False)).add_css_class("fieldset-normal-content"))
+            el_dl = elements.Fieldset(html.escape(
+                self.get_param_name_html_text(k))).add_css_class("fieldset-normal")
+            el_dl.append_child(elements.Pre(self.get_param_value_html_text(k, v)
+                                            ).add_css_class("fieldset-normal-content"))
             el_content.append_child(el_dl)
         return el_fs
 
@@ -320,6 +455,33 @@ class ReportTable(elements.Table):
             el_content.append_child(showview)
         return el_fs
 
+    def __build_user_screenshot_area(self, user_screenshots, label="用户截图"):
+
+        el_fs, el_item, el_title, el_content = self.__build_fieldset()
+        el_title_name = elements.Span(html.escape(label, False))
+
+        el_title.append_child(el_title_name)
+        el_item.add_css_class("seven-fieldset-item-hidden")
+        el_title_name.add_css_class("seven-testcase-screenshots")
+
+        for count, user_screenshot in enumerate(user_screenshots):
+            base64data = user_screenshot["base64data"]
+            filepath = user_screenshot["filepath"]
+            name = user_screenshot["name"]
+            if base64data:
+                src = "data:image/png;base64,{}".format(base64data)
+            else:
+                src = html.escape(filepath)
+            img_area = elements.Div()
+            showview = elements.Img().set_attr("onclick", 'show_image_on_new_window(this)').set_attr("src", src)
+            img_area.append_child(elements.P().append_child(
+                elements.Span(f"{count+1}")).append_child(elements.Span(name)))
+            img_area.append_child(showview)
+            img_area.add_css_class("user-screenshots-showarea")
+            showview.add_css_class("user-screenshot")
+            el_content.append_child(img_area)
+        return el_fs
+
     def __build_message_area(self, message_list, label="控制台信息"):
 
         el_fs, el_item, el_title, el_content = self.__build_fieldset()
@@ -339,6 +501,7 @@ class ReportTable(elements.Table):
         testdatas = testcase['testdatas']
         screenshot_info = testcase["screenshot_info"]
         attach_screenshot_to_report = screenshot_info.get('attach_screenshot_to_report', False)
+        user_screenshots = testcase["user_screenshots"]
         args, kwargs = testdatas
         nargs = {}
         for k, v in args.items():
@@ -353,12 +516,15 @@ class ReportTable(elements.Table):
         if nargs:
             show_div.append_child(self.__build_params_area(nargs, label="位置参数", param_type="args"))
         if kwargs:
-            show_div.append_child(self.__build_params_area(kwargs, label="关键字参数", param_type="kwargs"))
+            show_div.append_child(self.__build_params_area(
+                kwargs, label="关键字参数", param_type="kwargs"))
         if message[0] or message[1]:
             show_div.append_child(self.__build_message_area(message))
         show_div.append_child(self.__build_extra_info_area(testcase["extra_info"], label="基本信息"))
         if attach_screenshot_to_report:
-            show_div.append_child(self.__build_screenshot_area(screenshot_info, label="截图"))
+            show_div.append_child(self.__build_screenshot_area(screenshot_info, label="失败自动截图"))
+        if user_screenshots:
+            show_div.append_child(self.__build_user_screenshot_area(user_screenshots, label="用户截图"))
         cell.append_child(show_div)
         row.append_child(cell)
         return row
@@ -368,8 +534,10 @@ class ReportTable(elements.Table):
         rows = []
         for index, tc in enumerate(testcases):
 
-            tc_html_id = "%(tp_id)s%(sep)s%(prefix)s%(sn)s" % dict(tp_id=teststep_html_id, sep=self.ID_SEP, prefix=self.TESTCASE_ID_PREFIX, sn=(index + 1))
-            teststep_zone_html_id = "%(tc_id)s%(sep)s%(sn)s" % dict(tc_id=tc_html_id, sep=self.ID_SEP, sn=self.TESTSTEPS_ZONE_ROW_ID)
+            tc_html_id = "%(tp_id)s%(sep)s%(prefix)s%(sn)s" % dict(
+                tp_id=teststep_html_id, sep=self.ID_SEP, prefix=self.TESTCASE_ID_PREFIX, sn=(index + 1))
+            teststep_zone_html_id = "%(tc_id)s%(sep)s%(sn)s" % dict(
+                tc_id=tc_html_id, sep=self.ID_SEP, sn=self.TESTSTEPS_ZONE_ROW_ID)
             row = elements.TR().set_attr("id", tc_html_id).add_css_class(*self.TESTCASE_ROW_CSS_CLASS)
             row.set_attr("onclick", "toggleTestStepsRow('%s')" % teststep_zone_html_id)
             preceding_siblings = []
@@ -400,7 +568,8 @@ class ReportTable(elements.Table):
         for index, tp in enumerate(self._testpoints):
             tp_html_id = "%(prefix)s%(sn)s" % dict(prefix=self.TESTPOINT_ID_PREFIX, sn=(index + 1))
             row = elements.TR().set_attr("id", tp_html_id).add_css_class(self.TESTPOINT_ROW_CSS_CLASS)
-            row.set_attr("onclick", "toggleTestCaseOfTestPoint('%s','%s','%s',%s)" % (tp_html_id, self.TESTCASE_ID_PREFIX, self.ID_SEP, len(tp["testcases"])))
+            row.set_attr("onclick", "toggleTestCaseOfTestPoint('%s','%s','%s',%s)" %
+                         (tp_html_id, self.TESTCASE_ID_PREFIX, self.ID_SEP, len(tp["testcases"])))
             point_name = tp["name"]
             cls_name = point_name[0]
             chinese_name = point_name[1]
@@ -412,13 +581,20 @@ class ReportTable(elements.Table):
             name_cell.set_attr("title", html.escape(" ".join(point_name)))
             row.append_child(name_cell)
             row.append_child(elements.TD(tp["count"]).add_css_class(self.TESTPOINT_COUNT_CSS_CLASS))
-            row.append_child(elements.TD(tp["pass_count"]).add_css_class(self.TESTPOINT_PASS_CSS_CLASS))
-            row.append_child(elements.TD(tp["fail_count"]).add_css_class(self.TESTPOINT_FAIL_CSS_CLASS))
-            row.append_child(elements.TD(tp["block_count"]).add_css_class(self.TESTPOINT_BLOCK_CSS_CLASS))
-            row.append_child(elements.TD(tp["error_count"]).add_css_class(self.TESTPOINT_ERROR_CSS_CLASS))
-            row.append_child(elements.TD(tp["skip_count"]).add_css_class(self.TESTPOINT_SKIP_CSS_CLASS))
-            row.append_child(elements.TD(tp["xfail_count"]).add_css_class(self.TESTPOINT_XFAIL_CSS_CLASS))
-            row.append_child(elements.TD(tp["xpass_count"]).add_css_class(self.TESTPOINT_XPASS_CSS_CLASS))
+            row.append_child(elements.TD(tp["pass_count"]).add_css_class(
+                self.TESTPOINT_PASS_CSS_CLASS))
+            row.append_child(elements.TD(tp["fail_count"]).add_css_class(
+                self.TESTPOINT_FAIL_CSS_CLASS))
+            row.append_child(elements.TD(tp["block_count"]).add_css_class(
+                self.TESTPOINT_BLOCK_CSS_CLASS))
+            row.append_child(elements.TD(tp["error_count"]).add_css_class(
+                self.TESTPOINT_ERROR_CSS_CLASS))
+            row.append_child(elements.TD(tp["skip_count"]).add_css_class(
+                self.TESTPOINT_SKIP_CSS_CLASS))
+            row.append_child(elements.TD(tp["xfail_count"]).add_css_class(
+                self.TESTPOINT_XFAIL_CSS_CLASS))
+            row.append_child(elements.TD(tp["xpass_count"]).add_css_class(
+                self.TESTPOINT_XPASS_CSS_CLASS))
             row.after(*tuple(self._build_testcases_rows(tp_html_id, tp["testcases"])))
             rows.append(row)
         return rows
