@@ -13,9 +13,10 @@ import subprocess
 from ..utils import sutils
 from ..utils import attrs_manager
 from ..utils import attrs_marker
+from ..conf import image_show
 from ..core.errors import WindowNotFound
-from ..dm import DRIVER_MANAGER
-from ..dm import WIN_APP_DRIVER_HELPER
+from ..dm import driver_manager
+from ..dm import win_app_driver_helper
 from ..pylibs.lazy_libs import LazyLibs
 from ..utils.screenshot_capturer import ScreenshotCapturer
 
@@ -23,8 +24,9 @@ from ..utils.screenshot_capturer import ScreenshotCapturer
 class AbstractPage(attrs_manager.AttributeManager):
     """ 抽象页 """
 
-    DRIVER_MANAGER = attrs_marker.AttributeMarker(DRIVER_MANAGER, True, "Driver管理器")
-    WIN_APP_DRIVER_HELPER = attrs_marker.AttributeMarker(WIN_APP_DRIVER_HELPER, True, "启动和关闭WinAppDriver.exe助手")
+    DRIVER_MANAGER = attrs_marker.Const(driver_manager.DRIVER_MANAGER, "Driver管理器")
+    WIN_APP_DRIVER_HELPER = attrs_marker.Const(
+        win_app_driver_helper.WIN_APP_DRIVER_HELPER, "启动和关闭WinAppDriver.exe助手")
 
     def __init__(self, driver=None, alias=None, timeout=0.0, *args, **kwargs):
         """一般不建议创建页面的时候直接创建驱动实例，建议页面实例化后再调用页面提供的相关驱动实例化方法更好
@@ -37,7 +39,7 @@ class AbstractPage(attrs_manager.AttributeManager):
             kwargs: 见DriverManager.create_driver
         """
 
-        self._dm = self.__class__.DRIVER_MANAGER
+        self._dm: driver_manager.DriverManager = self.__class__.DRIVER_MANAGER
         self._dm.script_timeout = kwargs.pop("script_timeout", 5.0)
         self._dm.implicit_wait_timeout = kwargs.pop("implicit_wait_timeout", 0.0)
 
@@ -129,6 +131,9 @@ class AbstractPage(attrs_manager.AttributeManager):
     def open_window_app(self, remote_url="http://127.0.0.1:4723", desired_capabilities={}, alias=None, window_name=None, splash_delay=0, exact_match=True, desktop_alias=None):
         """ 创建 Windows 应用程序驱动程序会话
 
+        DriverManager类的`ENABLE_WINDOW_OPTIONS`属性为True时，会将`desired_capabilities` 转化为`WindowsOptions`实例,
+        desired_capabilities参数从appium 3. 版本开始被移除，如果你使用的appium 3.x 版本的，请将`ENABLE_WINDOW_OPTIONS`设为True
+
         Args:
             remote_url: WinAppDriver or Appium server url
             desired_capabilities:用于创建 Windows 应用程序驱动程序会话的功能
@@ -157,34 +162,57 @@ class AbstractPage(attrs_manager.AttributeManager):
                 self.sleep(splash_delay)
             return self.switch_window_app_by_name(remote_url, alias=alias, window_name=window_name, exact_match=exact_match, desktop_alias=desktop_alias, **desired_capabilities)
         self.driver_manager.open_desktop_session(remote_url, desktop_alias)
-        self.driver_manager.open_window_app(remote_url, alias=alias, desired_capabilities=desired_capabilities)
+
+        winapp_kwargs = {}
+        if self.driver_manager.ENABLE_WINDOW_OPTIONS:
+            winapp_kwargs[self.driver_manager.OPTIONS_PARAMETER] = self.driver_manager.cast_to_options(
+                desired_capabilities)
+        else:
+            winapp_kwargs["desired_capabilities"] = desired_capabilities
+        self.driver_manager.open_window_app(remote_url, alias=alias, **winapp_kwargs)
         return self
 
-    def switch_window_app_by_window_element(self, remote_url, window_element, alias=None, **kwargs):
+    def switch_window_app_by_window_element(self, remote_url, window_element, alias=None, **desired_capabilities):
+        """通过窗口元素切换应用
+
+        DriverManager类的`ENABLE_WINDOW_OPTIONS`属性为True时，会将`desired_capabilities` 转化为`WindowsOptions`实例,
+        desired_capabilities参数从appium 3. 版本开始被移除，如果你使用的appium 3.x 版本的，请将`ENABLE_WINDOW_OPTIONS`设为True
+        """
 
-        desired_caps = kwargs
         window_name = window_element.get_attribute("Name")
         if not window_name:
             msg = 'Error connecting webdriver to window "' + window_name + '". \n'
         else:
-            msg = 'Error connecting webdriver to window(which window element tag name is:{}). \n'.format(window_element.tag_name)
+            msg = 'Error connecting webdriver to window(which window element tag name is:{}). \n'.format(
+                window_element.tag_name)
         window = hex(int(window_element.get_attribute("NativeWindowHandle")))
-        if "app" in desired_caps:
-            del desired_caps["app"]
-        if "platformName" not in desired_caps:
-            desired_caps["platformName"] = "Windows"
-        if "forceMjsonwp" not in desired_caps:
-            desired_caps["forceMjsonwp"] = True
-        desired_caps["appTopLevelWindow"] = window
+        if "app" in desired_capabilities:
+            del desired_capabilities["app"]
+        if "platformName" not in desired_capabilities:
+            desired_capabilities["platformName"] = "Windows"
+        if "forceMjsonwp" not in desired_capabilities:
+            desired_capabilities["forceMjsonwp"] = True
+        desired_capabilities["appTopLevelWindow"] = window
+
+        winapp_kwargs = {}
+        if self.driver_manager.ENABLE_WINDOW_OPTIONS:
+            name = self.driver_manager.OPTIONS_PARAMETER
+            winapp_kwargs[name] = self.driver_manager.cast_to_options(desired_capabilities)
+        else:
+            winapp_kwargs["desired_capabilities"] = desired_capabilities
         try:
-            self.driver_manager.open_window_app(remote_url, alias=alias, desired_capabilities=desired_caps)
+            self.driver_manager.open_window_app(remote_url, alias=alias, **winapp_kwargs)
         except Exception as e:
             raise WindowNotFound(msg + str(e))
         return self
 
-    def switch_window_app_by_name(self, remote_url, window_name, alias=None, timeout=5, exact_match=True, desktop_alias=None, **kwargs):
+    def switch_window_app_by_name(self, remote_url, window_name, alias=None, timeout=5, exact_match=True, desktop_alias=None, **desired_capabilities):
+        """通过窗口名称切换应用
+
+        DriverManager类的`ENABLE_WINDOW_OPTIONS`属性为True时，会将`desired_capabilities` 转化为`WindowsOptions`实例,
+        desired_capabilities参数从appium 3. 版本开始被移除，如果你使用的appium 3.x 版本的，请将`ENABLE_WINDOW_OPTIONS`设为True
+        """
 
-        desired_caps = kwargs
         self.driver_manager.open_desktop_session(remote_url, desktop_alias)
         window_xpath = '//Window[contains(@Name, "' + window_name + '")]'
         window_locator = window_name
@@ -193,7 +221,6 @@ class AbstractPage(attrs_manager.AttributeManager):
                 window = self.find_element_by_name(window_locator)
             else:
                 window = self.find_element_by_xpath(window_xpath)
-            # print('Window_name "%s" found.' % window_name)
             window = hex(int(window.get_attribute("NativeWindowHandle")))
         except Exception:
             try:
@@ -201,24 +228,32 @@ class AbstractPage(attrs_manager.AttributeManager):
                     window = self.find_element_by_name(window_locator, timeout=timeout)
                 else:
                     window = self.find_element_by_xpath(window_xpath, timeout=timeout)
-                # print('Window_name "%s" found.' % window_name)
                 window = hex(int(window.get_attribute("NativeWindowHandle")))
             except Exception as e:
-                # print('Closing desktop session.')
-                raise LazyLibs.Selenium().exceptions.NoSuchWindowException('Error finding window "' + window_name + '" in the desktop session. ' 'Is it a top level window handle?' + '. \n' + str(e))
-        if "app" in desired_caps:
-            del desired_caps["app"]
-        if "platformName" not in desired_caps:
-            desired_caps["platformName"] = "Windows"
-        if "forceMjsonwp" not in desired_caps:
-            desired_caps["forceMjsonwp"] = True
-        desired_caps["appTopLevelWindow"] = window
+                msg = 'Error finding window "{}" in the desktop session. Is it a top level window handle? \n {}'
+                raise LazyLibs.Selenium().exceptions.NoSuchWindowException(msg.format(window_name, str(e)))
+        if "app" in desired_capabilities:
+            del desired_capabilities["app"]
+        if "platformName" not in desired_capabilities:
+            desired_capabilities["platformName"] = "Windows"
+        if "forceMjsonwp" not in desired_capabilities:
+            desired_capabilities["forceMjsonwp"] = True
+        desired_capabilities["appTopLevelWindow"] = window
+
+        winapp_kwargs = {}
+        if self.driver_manager.ENABLE_WINDOW_OPTIONS:
+            name = self.driver_manager.OPTIONS_PARAMETER
+            winapp_kwargs[name] = self.driver_manager.cast_to_options(desired_capabilities)
+        else:
+            winapp_kwargs["desired_capabilities"] = desired_capabilities
+
         # global application
         try:
             # print('Connecting to window_name "%s".' % window_name)
-            self.driver_manager.open_window_app(remote_url, alias=alias, desired_capabilities=desired_caps)
+            self.driver_manager.open_window_app(remote_url, alias=alias, **winapp_kwargs)
         except Exception as e:
-            raise WindowNotFound('Error connecting webdriver to window "' + window_name + '". \n' + str(e))
+            msg = 'Error connecting webdriver to window "{}" .\n {}'
+            raise WindowNotFound(msg.format(window_name, str(e)))
         return self
 
     def create_driver(self, driver_name, alias=None, *driver_args, **driver_kwargs):
@@ -244,22 +279,34 @@ class AbstractPage(attrs_manager.AttributeManager):
         return self
 
     def chrome(self, url=None, alias=None, **kwargs):
-        """
-        Creates a new instance of the chrome driver.
+        """Creates a new instance of the chrome driver.
 
         Starts the service and then creates new instance of chrome driver.
 
-        :Args:
-           kwargs:
-            - executable_path - path to the executable. If the default is used it assumes the executable is in the $PATH
-            - port - port you would like the service to run, if left as 0, a free port will be found.
-            - options - this takes an instance of ChromeOptions
-            - service_args - List of args to pass to the driver service
-            - desired_capabilities - Dictionary object with non-browser specific
-            capabilities only, such as "proxy" or "loggingPref".
-            - service_log_path - Where to log information from the driver.
-            - chrome_options - Deprecated argument for options
-            - keep_alive - Whether to configure ChromeRemoteConnection to use HTTP keep-alive.
+        Parameters
+        ----------
+        alias: 在缓存中驱动实例的别名，可通过该别名来切换驱动实例，如果在缓存中已存在则直接切换到该实例
+        kwargs : refer to the `selenium.webdriver.Chome`
+
+        Usage
+        -----
+        ```py
+        from pages.login_page import LoginPage
+
+        # selenium 3.x
+        from selenium import webdriver
+        options = webdriver.ChromeOptions()
+        chromedriver_path = r\"D:\\chromedriver-win64\\chromedriver.exe\"
+        LoginPage().chrome(executable_path=chromedriver_path, options=options)
+
+        # selenium 4.x
+        # executable_path has been deprecated, please pass in a Service object
+        # In Selenium 4, you’ll need to set the driver’s executable_path from a Service object to prevent deprecation warnings.
+        #  (Or don’t set the path and instead make sure that the driver you need is on the System PATH.)
+        from selenium.webdriver.chrome.service import Service as ChromeService
+        service = ChromeService(executable_path=chromedriver_path)
+        LoginPage().chrome(service=service, options=options)
+        ```
         """
         args = ()
         self.driver_manager.chrome(url=url, alias=alias, *args, **kwargs)
@@ -599,7 +646,8 @@ class AbstractPage(attrs_manager.AttributeManager):
             return driver.find_elements(by, locator)
         message = "{} with locator '{}' not found.".format(by, locator)
         try:
-            elements = LazyLibs.Selenium().WebDriverWait(driver, timeout).until(lambda x: x.find_elements(by, locator))
+            elements = LazyLibs.Selenium().WebDriverWait(
+                driver, timeout).until(lambda x: x.find_elements(by, locator))
         except LazyLibs.Selenium().exceptions.TimeoutException as t:
             message = message + "in {timeout}".format(timeout=timeout)
             screen = getattr(t, 'screen', None)
@@ -917,25 +965,151 @@ class AbstractPage(attrs_manager.AttributeManager):
         script = "arguments[0].click();"
         return self.execute_script(script, web_element)
 
-    def scroll_to(self, xpos, ypos):
-        """scroll to any position of an opened window of browser"""
+    def scroll_to(self, x: int, y: int, element=None):
+        """控制浏览器或元素滚动到指定坐标位置
 
-        js_code = "window.scrollTo(%s, %s);" % (xpos, ypos)
-        self.execute_script(js_code)
+        Args:
+            x (int): 水平滚动位置
+            y (int): 垂直滚动位置
+            element (WebElement, optional): 目标元素，None表示浏览器窗口
+        """
+        if element is None:
+            # 浏览器窗口滚动（兼容Firefox/Chrome）
+            self.execute_script("window.scrollTo({x}, {y});".format(x=x, y=y))
+        else:
+            # 元素内部滚动（兼容各版本）
+            self.execute_script("arguments[0].scrollTo({x}, {y});".format(x=x, y=y), element)
+        return self
 
-    def scroll_into_view(self, web_element):
+    def scroll_into_view(self, element):
 
         js_code = 'arguments[0].scrollIntoView();'
-        self.execute_script(js_code, web_element)
+        self.execute_script(js_code, element)
 
-    def scroll_to_bottom(self):
+    def scroll_to_top(self, element=None):
+        """
+        将浏览器窗口或元素滚动条滚动到顶部
 
-        bottom = self.execute_script("return document.body.scrollHeight;")
-        self.scroll_to(0, bottom)
+        Args:
+            element (WebElement, optional): 目标元素.
+                None表示浏览器窗口. Defaults to None.
 
-    def scroll_to_top(self):
+        Version:
+            Selenium 3.x/4.x通用实现
 
-        self.scroll_to(0, 0)
+        Example:
+            >>> scroll_to_top()  # 浏览器窗口滚动到顶部
+            >>> scroll_to_top(element)  # 元素滚动到顶部
+        """
+        if element is None:
+            # 浏览器窗口滚动到顶部
+            self.execute_script("window.scrollTo({top: 0, behavior: 'smooth'});")
+        else:
+            # 元素内部滚动到顶部
+            self.execute_script("arguments[0].scrollTo({top: 0, behavior: 'smooth'});", element)
+        return self
+
+    def scroll_to_bottom(self, element=None):
+        """
+        将浏览器窗口或元素滚动条滚动到底部
+
+        Args:
+            element (WebElement, optional): 目标元素.
+                None表示浏览器窗口. Defaults to None.
+        """
+        if element is None:
+            # 浏览器窗口滚动到底部
+            self.execute_script(
+                "window.scrollTo({top: document.body.scrollHeight, behavior: 'smooth'});")
+        else:
+            # 元素内部滚动到底部
+            self.execute_script(
+                "arguments[0].scrollTo({top: arguments[0].scrollHeight, behavior: 'smooth'});", element)
+        return self
+
+    def scroll_to_left(self, element=None):
+        """
+        将浏览器窗口或元素滚动条滚动到最左侧
+
+        Args:
+            element (WebElement, optional): 目标元素.
+                None表示浏览器窗口. Defaults to None.
+
+        Version:
+            Selenium 3.x/4.x通用实现
+
+        Example:
+            >>> scroll_to_left()  # 浏览器窗口滚动到最左
+            >>> scroll_to_left(element)  # 元素滚动到最左
+        """
+        if element is None:
+            # 浏览器窗口滚动到最左侧
+            self.execute_script("window.scrollTo(0, 0);")
+        else:
+            # 元素内部滚动到最左侧
+            self.execute_script("arguments[0].scrollLeft = 0;", element)
+        return self
+
+    def scroll_to_right(self, element=None):
+        """
+        滚动到浏览器窗口或元素最右侧
+
+        Args:
+            element (WebElement, optional): 目标元素.
+                None表示浏览器窗口. Defaults to None.
+
+        Version:
+            Selenium 3.x/4.x通用实现
+
+        Example:
+            >>> scroll_to_right()  # 浏览器窗口滚动到最右
+            >>> scroll_to_right(element)  # 元素滚动到最右
+        """
+        if element is None:
+            # 浏览器窗口滚动到最右侧
+            self.execute_script("window.scrollTo(document.body.scrollWidth, 0);")
+        else:
+            # 元素内部滚动到最右侧
+            self.execute_script("arguments[0].scrollLeft = arguments[0].scrollWidth;", element)
+        return self
+
+    def scroll_to_center(self, vertical=True, horizontal=False, element=None):
+        """
+        将浏览器窗口或元素滚动条滚动到中间位置
+
+        Args:
+            vertical (bool): 是否垂直滚动到中间. Defaults to True.
+            horizontal (bool): 是否水平滚动到中间. Defaults to False.
+            element (WebElement, optional): 目标元素. None表示浏览器窗口. Defaults to None.
+
+        Version:
+            Selenium 3.x/4.x通用实现
+
+        Example:
+            >>> scroll2center()  # 浏览器窗口垂直居中
+            >>> scroll2center(element=el, horizontal=True)  # 元素水平居中
+        """
+        js_template = """
+        var target = arguments[0] || window;
+        if(arguments[1]) {  // vertical
+            target.scrollTo({
+                top: target === window ?
+                    document.body.scrollHeight/2 - window.innerHeight/2 :
+                    arguments[0].scrollHeight/2 - arguments[0].clientHeight/2,
+                behavior: 'smooth'
+            });
+        }
+        if(arguments[2]) {  // horizontal
+            target.scrollTo({
+                left: target === window ?
+                    document.body.scrollWidth/2 - window.innerWidth/2 :
+                    arguments[0].scrollWidth/2 - arguments[0].clientWidth/2,
+                behavior: 'smooth'
+            });
+        }
+        """
+        self.execute_script(js_template, element, vertical, horizontal)
+        return self
 
     def drag_and_drop(self, source, target):
 
@@ -951,6 +1125,61 @@ class AbstractPage(attrs_manager.AttributeManager):
         """
         return ScreenshotCapturer.screenshot(file_name, self.driver)
 
+    def show2html(self, testcase, *, name="", path=None):
+        """截图并显示到html测试报告中
+
+        Parameters
+        ----------
+        testcase : 测试用例实例，显示在html测试报告中的哪个测试用例下
+        name : 在html报告中显示的名称
+        path : 截图保存路径
+        filepath : 截图文件完整路径
+
+        Usage
+        -----
+        ```
+        import stest
+        from erp_autotest.pages.login import LoginPage
+
+        class LoginTest(stest.AbstractTestCase):
+            @classmethod
+            def setUpClass(cls):
+
+                cls.url = "https://tv.cctv.com/live/cctv13"
+                cls.username = "zhangsan"
+                cls.password = "123456"
+
+            def setUp(self):
+                pass
+
+            @stest.Test(name="demo1", groups=["ss"])
+            def demo1(self):
+
+                page = LoginPage()
+                page.chrome().open_url(cls.url).actions.login(cls.username, cls.password).sleep(2)
+                page.show2html(self)
+                page.show2html(self, path="E:\\erp_autotest\\screenshots\\debug.png")
+
+            def tearDown(self):
+                pass
+
+            @classmethod
+            def tearDownClass(cls):
+                LoginPage.DRIVER_MANAGER.close_all_drivers()
+
+        if __name__ == "__main__":
+            LoginTest.run_test(buffer=True, argv=["python -m stest", "-g", "ss"])
+        ```
+        """
+        if path:
+            ScreenshotCapturer.screenshot(path, self.driver)
+            base64data = ""
+        else:
+            base64data = ScreenshotCapturer.screenshot_as_base64(self.driver)
+        image_show.show2html(testcase, base64data=base64data, filepath=path,
+                             name=name)
+        return self
+
     @property
     def session_id(self):
         """Returns the currently active browser session id"""
@@ -1251,6 +1480,10 @@ class AbstractPage(attrs_manager.AttributeManager):
         def __init__(self, page):
 
             self.page = page
+            self.init()
+
+        def init(self):
+            pass
 
         def sleep(self, seconds):
             """延时"""
@@ -1262,6 +1495,10 @@ class AbstractPage(attrs_manager.AttributeManager):
         def __init__(self, page):
 
             self.page = page
+            self.init()
+
+        def init(self):
+            pass
 
         def sleep(self, seconds):
             """延时"""
